/*************************************************************************
* File: Camera.cpp
* Author: Leo Nickerson, Timo Hohn
* Client: ECE 492 project
* Version: 1.00
* Last Modified: 09/04/2013
*
* Description:
* The following is a simple picture taking program using the StereoCamera
* and Camera classes made to be used with two Adafruit VC0706 camera.
* It takes pictures and stores the pictures on the host file system.
* Will only store up to 99 unique pictures
*
* Requirements:
*     RTOS Type - MicroC/OS-II
*     Periodic System Timer
*     UARTs and UART driver
*     Must be run in Debug Mode with hostfs enabled
*     Files with the proper names already present in hostfs
**************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "includes.h"
#include "StereoCamera.h"
#include "altera_avalon_uart.h"
#include "system.h"
#include "io.h"

// Address to write pictures to
#define RIGHT_ADDRESS (SDRAM_0_BASE+3*SDRAM_0_SPAN/4)
#define LEFT_ADDRESS (SDRAM_0_BASE+7*SDRAM_0_SPAN/8)

/* Definition of Task Stacks */
#define   TASK_STACKSIZE       2048
OS_STK    task1_stk[TASK_STACKSIZE];

/* Definition of Task Priorities */
#define TASK1_PRIORITY      1

/* Camera Control task */
void task1(void* pdata)
{
  printf("Hello World, I take pictures!\n");
  
  StereoCamera scam = StereoCamera("/dev/uart_0", "/dev/uart_1");

  // File names
  char leftFile[64];
  char rightFile[64];

  //Used for determining which file to write photos to
  int counter = 1;

  while(1){
	  //Only starts picture taking process when user enters v then enter
	  printf("press 'v' then enter to start\n");
	  while(fgetc(stdin)!='v');

	  printf("taking picture: ");
	  while(!scam.takePicture());  //freezes the video
	  printf("snap!\n");

	  FILE * picture0_fp, * picture1_fp;

	  // Opens the files
	  sprintf(rightFile, "/mnt/host/pictures/right%d%d.jpg", (counter/10), (counter%10));
	  sprintf(leftFile, "/mnt/host/pictures/left%d%d.jpg", (counter/10), (counter%10));

	  if((picture0_fp=fopen(rightFile, "r+"))==NULL)
	    printf("\tcould not open %s on host\n", rightFile);

	  if((picture1_fp=fopen(leftFile, "r+"))==NULL)
	    printf("\tcould not open %s on host\n", leftFile);

	  //Store pictures at specified addresses
	  printf("reading pictures...\n");
	  scam.readPicture(RIGHT_ADDRESS, LEFT_ADDRESS);

	  //Variables needed for writing to files
	  alt_u8 data;
	  size_t right_length;
	  while(!(right_length = scam.getFrameLength(0)));
	  size_t left_length;
	  while(!(left_length = scam.getFrameLength(1)));

	  //Writes pictures to files
	  for(size_t i=0; i<right_length; i++){
		  data = IORD_8DIRECT(RIGHT_ADDRESS, i);
		  fwrite(&data, sizeof(alt_u8), 1, picture0_fp);
	  }
	  for(size_t i=0; i<left_length; i++){
		  data = IORD_8DIRECT(LEFT_ADDRESS, i);
		  fwrite(&data, sizeof(alt_u8), 1, picture1_fp);
	  }
	  printf("done!\n");

	  //Resets the camera
	  scam.reset();
	  scam.setResLrg();

	  fclose(picture0_fp);
	  fclose(picture1_fp);

	  if(++counter >= 10)
		  counter = 1;
  }
}

/* The main function creates camera control task */
int main(void)
{

  OSTaskCreateExt(task1,
                  NULL,
                  (OS_STK *)&task1_stk[TASK_STACKSIZE-1],
                  TASK1_PRIORITY,
                  TASK1_PRIORITY,
                  task1_stk,
                  TASK_STACKSIZE,
                  NULL,
                  0);

  OSStart();
  return 0;
}

